package com.fzm.common.service.impl;

import cn.dev33.satoken.stp.StpUtil;
import cn.fzm.chain.simplesdk.client.ParaChainClient;
import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fzm.common.entity.AuthPerson;
import com.fzm.common.entity.User;
import com.fzm.common.enums.AuthStatusEnum;
import com.fzm.common.enums.AuthTypeEnum;
import com.fzm.common.enums.ResultCode;
import com.fzm.common.exception.GlobalException;
import com.fzm.common.mapper.UserMapper;
import com.fzm.common.params.LoginParam;
import com.fzm.common.properties.SmsProperties;
import com.fzm.common.service.AuthPersonService;
import com.fzm.common.service.UserService;
import com.fzm.common.utils.SmsUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;

/**
 * @author tangtuo
 * @date 2021/6/23 15:15
 */
@Service
@Transactional(rollbackFor = RuntimeException.class)
public class UserServiceImpl extends ServiceImpl<UserMapper, User> implements UserService {

    @Resource
    private PasswordEncoder passwordEncoder;

    @Resource
    private SmsProperties smsProperties;

    @Resource
    private SmsUtil smsUtil;

    @Resource
    private AuthPersonService authPersonService;

    @Resource
    private ParaChainClient paraClient;

    @Override
    public User loadUserByUsername(String username) {
        QueryWrapper<User> wrapper = new QueryWrapper<>();
        wrapper.eq("telephone", username).
                or().
                eq("email", username);
        return getOne(wrapper);
    }


    @Override
    public User login(LoginParam param) {
        String username = StringUtils.isNotBlank(param.getTelephone()) ?
                param.getTelephone() :
                param.getEmail();
        User user = loadUserByUsername(username);
        if (user == null) {
            // 当前手机号或者邮箱没有注册过,走注册逻辑
            User registerUser = new User();
            BeanUtil.copyProperties(param, registerUser, true);
            if (StringUtils.isNotBlank(param.getPassword())) {
                registerUser.setPassword(passwordEncoder.encode(param.getPassword()));
            } else {
                // 验证码注册流程
                if (!smsUtil.validateCode(getLoginCodeType(param.getCodetype()), username, param.getVerificationCode(), param.getCodetype())) {
                    throw GlobalException.newException(ResultCode.VALIDATE_FAILED, "验证码校验失败");
                }
            }
            save(registerUser);
            // 生成钱包
            String wallet = paraClient.walletCreateAccount(registerUser.getId().toString());
            if (StringUtils.isNotBlank(wallet)) {
                User u = new User().setId(registerUser.getId()).setWallet(wallet);
                updateById(u);
            }
            return getById(registerUser.getId());
        } else {
            // 登录逻辑
            if (StringUtils.isNotBlank(param.getPassword())) {
                // 密码登录流程
                if (!passwordEncoder.matches(param.getPassword(), user.getPassword())) {
                    throw GlobalException.newException(ResultCode.LOGIN_ERROR, "密码有误");
                }
            } else {
                // 验证码登录流程
                if (!smsUtil.validateCode(getLoginCodeType(param.getCodetype()), username, param.getVerificationCode(), param.getCodetype())) {
                    throw GlobalException.newException(ResultCode.VALIDATE_FAILED, "验证码校验失败");
                }
            }
            return user;
        }
    }

    @Override
    public User getUserByToken() {
        String token = StpUtil.getTokenValue();
        if (StringUtils.isBlank(token)) {
            throw GlobalException.newException(ResultCode.UNAUTHORIZED, "您还没有登录,请先登录");
        }
        Integer userId = Integer.parseInt(StpUtil.getLoginId().toString());
        return getById(userId);
    }


    public String getLoginCodeType(String type) {
        switch (type) {
            case "sms":
                return smsProperties.getMessageLoginCodetype();
            case "email":
                return smsProperties.getEmailLoginCodetype();
            case "voice":
                return smsProperties.getVoiceLoginCodetype();
            default:
                throw GlobalException.newException(ResultCode.CODE_ERROR);
        }
    }

    @Override
    public Boolean personAuth(AuthPerson authPerson) {
        Integer loginId = Integer.parseInt(StpUtil.getLoginId().toString());
        User user = new User().
                setAuthStatus(AuthStatusEnum.ONGOING.getStatus()).
                setAuthType(AuthTypeEnum.PERSON.getType()).
                setId(loginId);
        updateById(user);
        authPerson.setUserId(loginId);
        return authPersonService.save(authPerson);
    }

    @Override
    public User uploadAvatar(String avatar) {
        Integer loginId = Integer.parseInt(StpUtil.getLoginId().toString());
        User user = new User().
                setAvatar(avatar).
                setId(loginId);
        updateById(user);
        return getById(loginId);
    }
}
